/*
** cyclelog v1.2.3
**
** Cycle log files
**
** Copyright (c) 24.1.1996 by Andreas Ley <andreas.ley@kit.edu>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.05
** In this environment, neither lint -u nor gcc -Wall -fsyntax-only produce
** any messages. If you encounter any errors or need to make any changes to
** port it to another platform, please contact me.
**
** Version history
**
** Version 1.2.3 - 8.3.2024
**	Change name due to clash with offical debian package
**
** Version 1.2.2 - 10.9.1997
**	Return sensible exit codes
**
** Version 1.2.1 - 27.3.1996
**	Fixed umask/mode bug.
**
** Version 1.2 - 5.2.1996
**	Fixed versions decrease bug with multiple files.
**
** Version 1.1 - 25.1.1996
**	Check for versions above current <versions>, delete them too.
**
** Version 1.0 - 24.1.1996
**	Initial version
*/

static char copyright[] = "@(#)Copyright (C) 1996-2024 by Andreas Ley (andreas.ley@kit.edu)";
static char sccsid[] = "@(#)cyclelog v1.2.3 - Cycle log files";


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <limits.h>
#include <errno.h>

#ifndef FALSE
#define	FALSE	(0)		/* This is the naked Truth */
#define	TRUE	(1)		/* and this is the Light */
#endif


#define	DEFVERS	7		/* Default number of versions */

char	*image;
int	size=FALSE,versions=DEFVERS,owner=-1,group=-1,mode=-1,debug=0;



static int cyclelog(fname)
char	*fname;
{
	char		*f1,*f2;
	int		len,retval,version,fd;
	struct stat	buf;

	if (debug)
		(void)fprintf(stderr,"cyclelog(\"%s\")\n",fname);

	retval=stat(fname,&buf);
	if (retval) {
		(void)fprintf(stderr,"%s: can't stat ",image);
		perror(fname);
		return(1);
	}
	if (size&&buf.st_size==0)
		return(0);

	len=strlen(fname);
	f1=(char *)malloc((size_t)(len+10));
	(void)strcpy(f1,fname);
	(void)strcpy(f1+len,".");
	f2=(char *)malloc((size_t)(len+10));
	(void)strcpy(f2,fname);
	(void)strcpy(f2+len,".");
	len++;

	version=versions;
	do {
		(void)sprintf(f1+len,"%d",version++);
		if (debug)
			(void)printf("rm %s\n",f1);
		retval=remove(f1);
		if (retval&&errno!=ENOENT) {
			(void)fprintf(stderr,"%s: can't remove ",image);
			perror(f1);
			return(2);
		}
	} while(!retval);

	version=versions;
	(void)sprintf(f2+len,"%d",version);
	while (--version) {
		(void)sprintf(f1+len,"%d",version);
		if (debug)
			(void)printf("mv %s %s\n",f1,f2);
		retval=rename(f1,f2);
		if (retval&&errno!=ENOENT) {
			(void)fprintf(stderr,"%s: can't move %s to ",image,f1);
			perror(f2);
			return(2);
		}
		(void)strcpy(f2+len,f1+len);
	}

	if (debug)
		(void)printf("mv %s %s\n",fname,f2);
	retval=rename(fname,f2);
	if (retval&&errno!=ENOENT) {
		(void)fprintf(stderr,"%s: can't move %s to ",image,fname);
		perror(f2);
		return(2);
	}

	(void)umask(0);
	fd=open(fname,O_WRONLY|O_CREAT,mode<0?buf.st_mode:mode);
	if (fd<0) {
		(void)fprintf(stderr,"%s: can't open ",image);
		perror(fname);
		return(2);
	}
	retval=fchown(fd,owner<0?buf.st_uid:owner,group<0?buf.st_gid:group);
	if (retval) {
		(void)fprintf(stderr,"%s: can't chown ",image);
		perror(fname);
		return(2);
	}
	(void)close(fd);

	free((void *)f2);
	free((void *)f1);
	return(0);
}



static void usage()
{
	(void)fprintf(stderr,"Usage: %s [-s] [-c count] [-u user] [-g group] [-m mode] [filename...]\n",image);
	(void)fprintf(stderr,"-s  Cycle only if size of <filename> is greater than zero\n");
	(void)fprintf(stderr,"-c  Holds <count> old versions of <filename> (defaults to %d versions)\n",DEFVERS);
	(void)fprintf(stderr,"-u  Sets owner of new <filename> (defaults to current owner)\n");
	(void)fprintf(stderr,"-g  Sets group of new <filename> (defaults to current group)\n");
	(void)fprintf(stderr,"-m  Sets mode of new <filename> (defaults to current mode)\n");
	exit(1);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
	int		c,retval,maxretval;
	extern char	*optarg;
	extern int	optind;
	struct passwd	*pw;
	struct group	*gr;

	if ((image=strrchr(argv[0],'/')))
		image++;
	else
		image=argv[0];

	while ((c=getopt(argc,argv,"sc:u:g:m:Dvh?"))!=EOF)
		switch ((char)c) {
		case 's':
			size=TRUE;
			break;
		case 'c':
			versions=atoi(optarg);
			if (versions<1)
				usage();
			break;
		case 'u':
			pw=getpwnam(optarg);
			if (pw==NULL)
				usage();
			owner=pw->pw_uid;
			if (group<0)
				group=pw->pw_gid;
			break;
		case 'g':
			gr=getgrnam(optarg);
			if (gr==NULL)
				usage();
			group=gr->gr_gid;
			break;
		case 'm':
#ifdef sun
			mode=strtol(optarg,(char **)NULL,8);
#else
			mode=strtoul(optarg,(char **)NULL,8);
			if (mode==ULONG_MAX)
				usage();
#endif
			break;
		case 'D':
			debug++;
			break;
		case 'v':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
			exit(0);
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		case '?':
			usage();
		}

	maxretval=0;
	if (optind<argc)
		for (;optind<argc;optind++) {
			retval=cyclelog(argv[optind]);
			if (retval>maxretval)
				maxretval=retval;
		}
	else
		usage();

	return(maxretval);
}
